@@ -1100,6 +1100,8 @@ convert_to_aggr (type, expr, msgp, protect)
 		     actualparameterlist (NULL_TREE, TYPE_ARG_TYPES (fntype), parmlist, NULL_TREE, LOOKUP_NORMAL),
 		     NULL_TREE);
   TREE_TYPE (result) = TREE_TYPE (fntype);
+  TREE_VOLATILE (result) = 1;
+  TREE_RAISES (result) = !! TYPE_RAISES_EXCEPTIONS (fntype);
   return result;
 }
 
@@ -1166,7 +1168,13 @@ convert (type, expr)
       return error_mark_node;
     }
   if (code == VOID_TYPE)
-    return build1 (CONVERT_EXPR, type, e);
+    {
+      tree rval = build_type_conversion (NOP_EXPR, type, e, 0);
+      /* If we can convert to void type via a type conversion, do so.  */
+      if (rval)
+	return rval;
+      return build1 (CONVERT_EXPR, type, e);
+    }
 #if 0
   /* This is incorrect.  A truncation can't be stripped this way.
      Extensions will be stripped by the use of get_unwidened.  */
@@ -1224,7 +1232,8 @@ convert (type, expr)
 
 	  if (TYPE_HAS_CONSTRUCTOR (type))
 	    {
-	      tree rval = build_method_call (NULL_TREE, DECL_NAME (TYPE_NAME (type)), build_tree_list (NULL_TREE, e), CLASSTYPE_AS_LIST (type), 0);
+	      tree rval = build_method_call (NULL_TREE, DECL_NAME (TYPE_NAME (type)), build_tree_list (NULL_TREE, e), CLASSTYPE_AS_LIST (type),
+					     conversion ? LOOKUP_NO_CONVERSION : 0);
 
 	      if (rval != error_mark_node)
 		{
@@ -1471,6 +1480,8 @@ build_type_conversion (code, xtype, expr, for_sure)
   /* No perfect match found, try default.  */
   if (TREE_CODE (type) == POINTER_TYPE)
     type_default = ptr_type_node;
+  else if (type == void_type_node)
+    return NULL_TREE;
   else
     {
       extern tree default_conversion ();
