@@ -60,7 +60,286 @@ static int global_temp_name_counter;
 
 /* The (assembler) name of the first globally-visible object output.  */
 extern char * first_global_object_name;
+
+/* C (and C++) language-specific option variables.  */
 
+/* Nonzero means allow type mismatches in conditional expressions;
+   just make their values `void'.   */
+
+int flag_cond_mismatch;
+
+/* Nonzero means don't recognize the keyword `asm'.  */
+
+int flag_no_asm;
+
+/* Nonzero means do some things the same way PCC does.  */
+
+int flag_traditional;
+
+/* Nonzero means warn about implicit declarations.  */
+
+int warn_implicit = 1;
+
+/* Nonzero means warn about function definitions that default the return type
+   or that use a null return and have a return-type other than void.  */
+
+int warn_return_type;
+
+/* Nonzero means give string constants the type `const char *'
+   to get extra warnings from them.  These warnings will be too numerous
+   to be useful, except in thoroughly ANSIfied programs.  */
+
+int warn_write_strings;
+
+/* Nonzero means warn about pointer casts that can drop a type qualifier
+   from the pointer target type.  */
+
+int warn_cast_qual;
+
+/* Nonzero means warn about sizeof(function) or addition/subtraction
+   of function pointers.  */
+
+int warn_pointer_arith;
+
+/* Nonzero means warn for all old-style non-prototype function decls.  */
+
+int warn_strict_prototypes;
+
+/* Non-zero means warn in function declared in derived class has the
+   same name as a virtual in the base class, but fails to match the
+   type signature of any virtual function in the base class.  */
+int warn_overloaded_virtual;
+
+/* Non-zero means warn when converting between different enumeral types.  */
+int warn_enum_clash;
+
+/* Nonzero means `$' can be in an identifier.
+   See cccp.c for reasons why this breaks some obscure ANSI C programs.  */
+
+#ifndef DOLLARS_IN_IDENTIFIERS
+#define DOLLARS_IN_IDENTIFIERS 0
+#endif
+int dollars_in_ident = DOLLARS_IN_IDENTIFIERS;
+
+/* Nonzero for -no-strict-prototype switch: do not consider empty
+   argument prototype to mean function takes no arguments.  */
+
+int strict_prototype = 1;
+int strict_prototypes_lang_c, strict_prototypes_lang_cplusplus = 1;
+
+/* Nonzero means that labels can be used as first-class objects */
+
+int flag_labels_ok;
+
+/* Non-zero means to collect statistics which might be expensive
+   and to print them when we are done.  */
+int flag_detailed_statistics;
+
+/* C++ specific flags.  */   
+/* Nonzero for -fall-virtual: make every member function (except
+   constructors) lay down in the virtual function table.  Calls
+   can then either go through the virtual function table or not,
+   depending.  */
+
+int flag_all_virtual;
+
+/* Zero means that `this' is a *const.  This gives nice behavior in the
+   2.0 world.  Nonzero gives 1.2-compatible behavior.  */
+
+int flag_this_is_variable;
+
+/* Nonzero means memoize our member lookups.  */
+
+int flag_memoize_lookups; int flag_save_memoized_contexts;
+
+/* Nonzero means to implement `dynamic' features a la SOS.  */
+
+int flag_dynamic;
+
+/* 2 means write out only specific virtual function tables
+   and give them (C) public visibility.
+   1 means write out virtual function tables and give them
+   (C) public visibility.
+   0 means write out virtual function tables and give them
+   (C) static visibility (default).
+   -1 means declare virtual function tables extern.  */
+
+int write_virtuals;
+
+/* Nonzero means we should attempt to elide constructors when possible.  */
+
+int flag_elide_constructors;
+
+/* Nonzero means if the type has methods, only output debugging
+   information if methods are actually written to the asm file.  */
+
+int flag_minimal_debug;
+
+/* Nonzero means recognize and handle exception handling constructs.
+   2 means handle exceptions the way Spring wants them handled.  */
+
+int flag_handle_exceptions;
+
+/* Nonzero means that member functions defined in class scope are
+   inline by default.  */
+
+int flag_default_inline = 1;
+
+/* Nonzero means that functions declared `inline' will be treated
+   as `static'.  Used in conjunction with -g.  */
+int flag_no_inline = 0;
+
+/* Controls whether enums and ints freely convert.
+   1 means with complete freedom.
+   0 means enums can convert to ints, but not vice-versa.  */
+int flag_int_enum_equivalence;
+
+/* Table of language-dependent -f options.
+   STRING is the option name.  VARIABLE is the address of the variable.
+   ON_VALUE is the value to store in VARIABLE
+    if `-fSTRING' is seen as an option.
+   (If `-fno-STRING' is seen as an option, the opposite value is stored.)  */
+
+static struct { char *string; int *variable; int on_value;} lang_f_options[] =
+{
+  {"signed-char", &flag_signed_char, 1},
+  {"unsigned-char", &flag_signed_char, 0},
+  {"short-enums", &flag_short_enums, 1},
+  {"cond-mismatch", &flag_cond_mismatch, 1},
+  {"asm", &flag_no_asm, 0},
+  {"labels-ok", &flag_labels_ok, 1},
+  {"stats", &flag_detailed_statistics, 1},
+  {"this-is-variable", &flag_this_is_variable, 1},
+  {"strict-prototype", &strict_prototypes_lang_cplusplus, 1},
+  {"warn-implicit", &warn_implicit, 1},
+  {"all-virtual", &flag_all_virtual, 1},
+  {"memoize-lookups", &flag_memoize_lookups, 1},
+  {"elide-constructors", &flag_elide_constructors, 1},
+  {"minimal-debug", &flag_minimal_debug, 1},
+  {"handle-exceptions", &flag_handle_exceptions, 1},
+  {"spring-exceptions", &flag_handle_exceptions, 2},
+  {"default-inline", &flag_default_inline, 1},
+  {"inline", &flag_no_inline, 0},
+  {"dollars-in-identifiers", &dollars_in_ident, 1},
+  {"enum-int-equiv", &flag_int_enum_equivalence, 1},
+};
+
+/* Decode the string P as a language-specific option.
+   Return 1 if it is recognized (and handle it);
+   return 0 if not recognized.  */
+
+int   
+lang_decode_option (p)
+     char *p;
+{
+  if (!strcmp (p, "-ftraditional") || !strcmp (p, "-traditional"))
+    flag_traditional = 1, dollars_in_ident = 1, flag_writable_strings = 1,
+    flag_this_is_variable = 1;
+  /* The +e options are for cfront compatability.  */
+  else if (p[0] == '+' && p[1] == 'e')
+    {
+      int old_write_virtuals = write_virtuals;
+      if (p[2] == '1')
+	write_virtuals = 1;
+      else if (p[2] == '0')
+	write_virtuals = -1;
+      else if (p[2] == '2')
+	write_virtuals = 2;
+      else error ("invalid +e option");
+      if (old_write_virtuals != 0
+	  && write_virtuals != old_write_virtuals)
+	error ("conflicting +e options given");
+    }
+  else if (p[0] == '-' && p[1] == 'f')
+    {
+      /* Some kind of -f option.
+	 P's value is the option sans `-f'.
+	 Search for it in the table of options.  */
+      int found = 0, j;
+
+      p += 2;
+      /* Try special -f options.  */
+
+      if (!strcmp (p, "save-memoized"))
+	{
+	  flag_memoize_lookups = 1;
+	  flag_save_memoized_contexts = 1;
+	  found = 1;
+	}
+      else if (!strcmp (p, "SOS"))
+	{
+	  flag_all_virtual = 2;
+	  found = 1;
+	}
+      else for (j = 0;
+		!found && j < sizeof (lang_f_options) / sizeof (lang_f_options[0]);
+		j++)
+	{
+	  if (!strcmp (p, lang_f_options[j].string))
+	    {
+	      *lang_f_options[j].variable = lang_f_options[j].on_value;
+	      /* A goto here would be cleaner,
+		 but breaks the vax pcc.  */
+	      found = 1;
+	    }
+	  if (p[0] == 'n' && p[1] == 'o' && p[2] == '-'
+	      && ! strcmp (p+3, lang_f_options[j].string))
+	    {
+	      *lang_f_options[j].variable = ! lang_f_options[j].on_value;
+	      found = 1;
+	    }
+	}
+      return found;
+    }
+  else if (p[0] == '-' && p[1] == 'W')
+    {
+      /* The -W options control the warning behavior of the compiler.  */
+      p += 2;
+
+      if (!strcmp (p, "implicit"))
+	warn_implicit = 1;
+      else if (!strcmp (p, "return-type"))
+	warn_return_type = 1;
+      else if (!strcmp (p, "write-strings"))
+	warn_write_strings = 1;
+      else if (!strcmp (p, "cast-qual"))
+	warn_cast_qual = 1;
+      else if (!strcmp (p, "pointer-arith"))
+	warn_pointer_arith = 1;
+      else if (!strcmp (p, "strict-prototypes"))
+	warn_strict_prototypes = 1;
+      else if (!strcmp (p, "comment"))
+	;			/* cpp handles this one.  */
+      else if (!strcmp (p, "comments"))
+	;			/* cpp handles this one.  */
+      else if (!strcmp (p, "trigraphs"))
+	;			/* cpp handles this one.  */
+      else if (!strcmp (p, "all"))
+	{
+	  extra_warnings = 1;
+	  warn_implicit = 1;
+	  warn_return_type = 1;
+	  warn_unused = 1;
+	  warn_switch = 1;
+#if 0
+	  warn_enum_clash = 1;
+#endif
+	}
+
+      else if (!strcmp (p, "overloaded-virtual"))
+	warn_overloaded_virtual = 1;
+      else if (!strcmp (p, "enum-clash"))
+	warn_enum_clash = 1;
+      else return 0;
+    }
+  else if (!strcmp (p, "-ansi"))
+    flag_no_asm = 1, dollars_in_ident = 0;
+  else
+    return 0;
+
+  return 1;
+}
+
 /* Incorporate `const' and `volatile' qualifiers for member functions.
    FUNCTION is a TYPE_DECL or a FUNCTION_DECL.
    QUALS is a list of qualifiers.  */
