@@ -65,9 +65,9 @@ tree current_exception_object;
 /* Low-level rtl interface.  */
 #include "rtl.h"
 
-/* Cache `setjmp', `longjmp', and `abort' after default conversion.
-   Maybe later they will get built-in.  */
-static tree BISJ, BILJ, BIA;
+/* Cache `setjmp', `longjmp', and `unhandled_exception' after
+   default conversion.  Maybe later they will get built-in.  */
+static tree BISJ, BILJ, BIUE;
 
 /* Local variables which give the appearance that exception
    handling is part of the language and the execution model.  */
@@ -449,23 +449,29 @@ end_exception_decls ()
 /* Statement-level exception semantics.  */
 
 void
-cplus_expand_start_try ()
+cplus_expand_start_try (implicit)
+     int implicit;
 {
   tree call_to_setjmp;
   tree handler, ref;
 
   /* Start a new block enclosing the whole handler.  */
-  pushlevel (0);
-  clear_last_expr ();
-  push_momentary ();
+  if (implicit)
+    pushlevel_temporary (1);
+  else
+    {
+      pushlevel (0);
+      clear_last_expr ();
+      push_momentary ();
 
-  /* Encompass whole exception handler in one big binding contour.
-     If RAISE should throw out of the whole TRY/EXCEPT block, call
-     `expand_start_bindings' with argument of 1.  */
-  expand_start_bindings (0);
+      /* Encompass whole exception handler in one big binding contour.
+	 If RAISE should throw out of the whole TRY/EXCEPT block, call
+	 `expand_start_bindings' with argument of 1.  */
+      expand_start_bindings (0);
+    }
 
   /* Allocate handler in that block.  It's real name will come later.
-     Note that it will be the only name in this binding contour.  */
+     Note that it will be the first name in this binding contour.  */
   handler = get_temp_name (EHS_type, 0);
   DECL_INITIAL (handler) = error_mark_node;
   finish_decl (handler, NULL_TREE, 0);
@@ -478,32 +484,48 @@ cplus_expand_start_try ()
   expand_start_try (build_binary_op (EQ_EXPR, call_to_setjmp, integer_zero_node), 0, 1);
 }
 
+/* If KEEP is 1, then declarations in the TRY statement are worth keeping.
+   If KEEP is 2, then the TRY statement was generated by the compiler.
+   If KEEP is 0, the declarations in the TRY statement contain errors.  */
+
 tree
 cplus_expand_end_try (keep)
      int keep;
 {
-  tree decls = getdecls ();
+  tree decls;
   tree decl;
 
+  if (keep < 2)
+    pop_implicit_try_blocks (NULL_TREE);
+
+  decls = getdecls ();
+
   /* Emit code to avoid falling through into a default
      handler that might come later.  */
   expand_end_try ();
 
-  /* Pops binding contour local to TRY.  */
-  if (keep)
-    {
-      expand_end_bindings (decls, decls != 0, 1);
-      poplevel (decls != 0, 1, 0);
-    }
-  else
+  /* Pops binding contour local to TRY, and get the exception handler
+     object built by `...start_try'.  */
+  switch (keep)
     {
+    case 0:
       expand_end_bindings (decls, 0, 1);
       poplevel (0, 0, 0);
-    }
-  pop_momentary (); 
+      pop_momentary (); 
+      decl = getdecls ();
+      break;
 
-  /* Get the exception handler object built by `...start_try'.  */
-  decl = getdecls ();
+    case 1:
+      expand_end_bindings (decls, decls != 0, 1);
+      poplevel (decls != 0, 1, 0);
+      pop_momentary ();
+      decl = getdecls ();
+      break;
+
+    default:
+      decl = tree_last (decls);
+      break;
+    }
 
   assert (TREE_CODE (decl) == VAR_DECL && TREE_TYPE (decl) == EHS_type);
 
@@ -538,13 +560,24 @@ cplus_expand_start_except (name, decl)
   expand_assignment (EHS_decl, EHS_prev, 0, 0);
 }
 
+/* Generate the call to `unhandled_exception' that is appropriate
+   for this particular unhandled exception.  */
+static tree
+call_to_unhandled_exception ()
+{
+  extern int lineno;
+  tree parms = tree_cons (NULL_TREE,
+			  combine_strings (build_string (strlen (input_filename), input_filename)),
+			  build_tree_list (NULL_TREE, build_int_2 (lineno, 0)));
+  return build_function_call (BIUE, parms);
+}
+
 /* Note that this must be mirror image of `...start_try'.
    DFAULT is the default clause, if there was one.
-   EXITFLAG is nonzero if we should try an outer handler.  */
+   DFAULT is ERROR_MARK_NODE when this ends an implicit handler.  */
 void
-cplus_expand_end_except (dfault, exitflag)
+cplus_expand_end_except (dfault)
      tree dfault;
-     int exitflag;
 {
   extern tree expand_end_except (); /* stmt.c.  */
   tree decls, raised;
@@ -555,27 +588,26 @@ cplus_expand_end_except (dfault, exitflag)
 	 reraise the exception.  Otherwise, generate code to call `abort'.  */
       if (in_try_block (1) == 0)
 	{
-	  tree call_to_abort;
 	  expand_start_cond (build (EQ_EXPR, integer_type_node,
 				    exception_throw_decl, integer_zero_node), 0);
-	  call_to_abort = build_function_call (BIA, NULL_TREE);
-	  expand_expr (call_to_abort, 0, VOIDmode, 0);
+	  expand_expr (call_to_unhandled_exception (), 0, VOIDmode, 0);
 	  expand_end_cond ();
 	}
     }
-  if (exitflag)
-    {
-      /* Try the next handler.  */
-      if (! expand_escape_except ())
-	compiler_error ("except nesting botch");
-    }
+
+  /* Try the next handler.  */
+  if (! expand_escape_except ())
+    compiler_error ("except nesting botch");
 
   raised = expand_end_except ();
 
   decls = getdecls ();
   expand_end_bindings (decls, decls != 0, 1);
   poplevel (decls != 0, 1, 0);
-  pop_momentary ();
+
+  /* Implicit handlers do not use the momentary obstack.  */
+  if (dfault != error_mark_node)
+    pop_momentary ();
 
   if (! in_try_block (1))
     {
@@ -587,13 +619,13 @@ cplus_expand_end_except (dfault, exitflag)
 	  raised = TREE_CHAIN (raised);
 	}
     }
-  else if (dfault == NULL_TREE)
+  else if (dfault == NULL_TREE || dfault == error_mark_node)
     {
       expand_start_cond (build (NE_EXPR, integer_type_node,
 				exception_throw_decl,
 				integer_zero_node), 0);
       /* We fell off the end of this try block.  Try going to the next.
-	 The exit_label will be the beginning of the next try block.  */
+	 The escape_label will be the beginning of the next try block.  */
       if (! expand_escape_except ())
 	compiler_error ("except nesting botch");
       expand_end_cond ();
@@ -728,7 +760,8 @@ expand_cplus_start_catch (raise_id)
 }
 
 void
-expand_cplus_end_catch ()
+expand_cplus_end_catch (for_reraise)
+     int for_reraise;
 {
   if (current_exception_type
       && TYPE_NEEDS_DESTRUCTOR (current_exception_type))
@@ -736,8 +769,16 @@ expand_cplus_end_catch ()
       /* Destroy the specific exception object now in scope.  */
       expand_end_bindings (getdecls (), 1, 1);
     }
-  if (! expand_escape_except ())
-    abort ();
+  if (for_reraise)
+    {
+      if (! expand_escape_except ())
+	abort ();
+    }
+  else
+    {
+      if (! expand_end_catch ())
+	abort ();
+    }
   expand_end_cond ();
 }
 
@@ -745,9 +786,38 @@ void
 expand_cplus_reraise (exceptions)
      tree exceptions;
 {
-  tree current_exception_ptr = build (NOP_EXPR, NULL_TREE,
-				      TREE_OPERAND (current_exception_object, 0));
+  tree current_exception_ptr;
 
+  /* reraise ALL, used by compiler.  */
+  if (exceptions == NULL_TREE)
+    {
+      /* Now treat reraise like catch/raise.  */
+      expand_catch (error_mark_node);
+      expand_raise (error_mark_node);
+      expand_assignment (EHS_name,
+			 build (COMPONENT_REF, ptr_type_node,
+				current_exception_decl, TREE_OPERAND (EHS_name, 1)), 0, 0);
+      expand_assignment (EHS_parms,
+			 build (COMPONENT_REF, ptr_type_node,
+				current_exception_decl, TREE_OPERAND (EHS_parms, 1)), 0, 0);
+      if (in_try_block (1) == 0)
+	{
+	  sets_exception_throw_decl = 1;
+	  expand_assignment (exception_throw_decl, integer_one_node, 0, 0);
+	}
+      /* Set to zero so that destructor will not be called.  */
+      expand_assignment (build (NOP_EXPR, ptr_type_node,
+				TREE_OPERAND (current_exception_object, 0)),
+			 integer_zero_node, 0, 0);
+      if (! expand_escape_except ())
+	abort ();
+      return;
+    }
+
+  current_exception_ptr = build (NOP_EXPR, NULL_TREE,
+				 TREE_OPERAND (current_exception_object, 0));
+
+  /* reraise from a list of exceptions.  */
   while (exceptions)
     {
       tree type = lookup_exception_type (current_class_type, current_class_name,
@@ -774,11 +844,11 @@ expand_cplus_reraise (exceptions)
       /* Set to zero so that destructor will not be called.  */
       if (TREE_TYPE (current_exception_ptr) != error_mark_node)
 	expand_assignment (current_exception_ptr, integer_zero_node, 0, 0);
-      expand_cplus_end_catch (exceptions);
+      expand_cplus_end_catch (1);
       exceptions = TREE_CHAIN (exceptions);
     }
   /* Don't propagate any unhandled exceptions.  */
-  expand_expr (build_function_call (BIA, NULL_TREE), 0, VOIDmode, 0);
+  expand_expr (call_to_unhandled_exception (), 0, VOIDmode, 0);
 }
 
 void
@@ -807,8 +877,7 @@ setup_exception_throw_decl ()
 void
 init_exception_processing ()
 {
-  extern tree abort_fndecl;
-
+  extern tree unhandled_exception_fndecl;
   tree cname = get_identifier ("ExceptionHandler");
   tree field, chain;
   tree ctor, dtor;
@@ -834,7 +903,7 @@ init_exception_processing ()
 									  hash_tree_chain (integer_type_node, NULL_TREE))),
 				    NOT_BUILT_IN, 0);
   BILJ = default_conversion (longjmp_fndecl);
-  BIA = default_conversion (abort_fndecl);
+  BIUE = default_conversion (unhandled_exception_fndecl);
 
   pop_lang_context ();
 
