@@ -45,20 +45,7 @@ extern void free ();
 void init_search ();
 extern struct obstack *current_obstack;
 
-/* Stack of places to restore the search obstack back to.  */
-   
-struct search_level
-{
-  /* Pointer back to previous such level.  */
-  struct search_level *prev;
-  /* Point to obstack we shoudl return to.  */
-  struct obstack *obstack;
-  /* First place we start putting data.  */
-  tree *first;
-  /* Number of entries we can have from `first'.
-     Right now we are dumb: if we overflow, abort.  */
-  int limit;
-};
+#include "stack.h"
 
 /* Obstack used for remembering decision points of breadth-first.  */
 static struct obstack search_obstack;
@@ -66,6 +53,36 @@ static struct obstack search_obstack;
 /* Obstack used to bridge from one function context to another.  */
 static struct obstack bridge_obstack;
 
+/* Methods for pushing and popping objects to and from obstacks.  */
+
+struct stack_level *
+push_stack_level (obstack, tp, size)
+     struct obstack *obstack;
+     void *tp;
+     int size;
+{
+  struct stack_level *stack;
+  stack = (struct stack_level *) obstack_next_free (obstack);
+  obstack_grow (obstack, tp, size);
+  obstack_finish (obstack);
+  stack->obstack = obstack;
+  stack->first = (tree *) obstack_base (obstack);
+  stack->limit = obstack_room (obstack) / sizeof (tree *);
+  return stack;
+}
+
+struct stack_level *
+pop_stack_level (stack)
+     struct stack_level *stack;
+{
+  struct stack_level *tem = stack;
+  struct obstack *obstack = tem->obstack;
+  stack = tem->prev;
+  obstack_free (obstack, tem);
+  return stack;
+}
+
+#define search_level stack_level
 static struct search_level *search_stack;
 
 static tree lookup_field_1 ();
@@ -74,52 +91,32 @@ static int lookup_fnfields_1 ();
 /* Allocate a level of searching.  */
 static struct search_level *
 push_search_level (stack, obstack)
-     struct search_level *stack;
+     struct stack_level *stack;
      struct obstack *obstack;
 {
   struct search_level tem;
-
   tem.prev = stack;
-  stack = (struct search_level *) obstack_next_free (obstack);
-  obstack_grow (obstack, &tem, sizeof (tem));
-  obstack_finish (obstack);
-  stack->obstack = obstack;
-  stack->first = (tree *) obstack_base (obstack);
-  stack->limit = obstack_room (obstack) / sizeof (tree *);
-  return stack;
+
+  return push_stack_level (obstack, &tem, sizeof (tem));
 }
 
 /* Discard a level of search allocation.  */
-
-static struct search_level *
-pop_search_level (stack)
-     struct search_level *stack;
-{
-  struct search_level *tem = stack;
-  struct obstack *obstack = tem->obstack;
-  stack = tem->prev;
-  obstack_free (obstack, tem);
-  return stack;
-}
+#define pop_search_level pop_stack_level
 
 /* Search memoization.  */
 struct type_level
 {
-  /* Pointer back to previous such level.  */
-  struct type_level *prev;
+  struct stack_level base;
+
   /* First object allocated in obstack of entries.  */
   char *entries;
+
   /* Number of types memoized in this context.  */
   int len;
+
   /* Type being memoized; save this if we are saving
      memoized contexts.  */
   tree type;
-
-  /* First place we start putting data.  */
-  tree *first;
-  /* Number of entries we can have from `first'.
-     Right now we are dumb: if we overflow, abort.  */
-  int limit;
 };
 
 /* Obstack used for memoizing member and member function lookup.  */
@@ -153,36 +150,29 @@ static struct type_level *prev_type_stack;
 /* Allocate a level of type memoziation context.  */
 static struct type_level *
 push_type_level (stack, obstack)
-     struct type_level *stack;
+     struct stack_level *stack;
      struct obstack *obstack;
 {
   struct type_level tem;
 
-  tem.prev = stack;
+  tem.base.prev = stack;
+
   obstack_finish (&type_obstack_entries);
   tem.entries = (char *) obstack_base (&type_obstack_entries);
   tem.len = 0;
   tem.type = NULL_TREE;
-  stack = (struct type_level *) obstack_next_free (obstack);
-  obstack_grow (obstack, &tem, sizeof (tem));
-  obstack_finish (obstack);
-  stack->first = (tree *) obstack_base (obstack);
-  stack->limit = obstack_room (obstack) / sizeof (tree *);
-  return stack;
+
+  return (struct type_level *)push_stack_level (obstack, &tem, sizeof (tem));
 }
 
 /* Discard a level of type memoziation context.  */
 
 static struct type_level *
-pop_type_level (stack, obstack)
+pop_type_level (stack)
      struct type_level *stack;
-     struct obstack *obstack;
 {
-  struct type_level *tem = stack;
-  stack = tem->prev;
-  obstack_free (&type_obstack_entries, tem->entries);
-  obstack_free (obstack, tem);
-  return stack;
+  obstack_free (&type_obstack_entries, stack->entries);
+  return (struct type_level *)pop_stack_level (stack);
 }
 
 /* Make something that looks like a TREE_LIST, but
@@ -308,14 +298,14 @@ push_memoized_context (type, use_old)
 	  type_stack = prev_type_stack;
 	  prev_type_stack = 0;
 
-	  tem = &type_stack->first[0];
+	  tem = &type_stack->base.first[0];
 	  len = type_stack->len;
 	  while (len--)
 	    CLASSTYPE_MTABLE_ENTRY (tem[len*2]) = tem[len*2+1];
 	  return;
 	}
       /* Otherwise, need to pop old stack here.  */
-      type_stack = pop_type_level (prev_type_stack, &type_obstack);
+      type_stack = pop_type_level (prev_type_stack);
       prev_type_memoized = 0;
       prev_type_stack = 0;
     }
@@ -333,7 +323,7 @@ pop_memoized_context (use_old)
      int use_old;
 {
   int len;
-  tree *tem = &type_stack->first[0];
+  tree *tem = &type_stack->base.first[0];
 
   if (! flag_save_memoized_contexts)
     use_old = 0;
@@ -355,9 +345,9 @@ pop_memoized_context (use_old)
 	  = MEMOIZED_CHAIN (CLASSTYPE_MTABLE_ENTRY (tem[len*2]));
     }
   if (! use_old)
-    type_stack = pop_type_level (type_stack, &type_obstack);
+    type_stack = pop_type_level (type_stack);
   else
-    type_stack = type_stack->prev;
+    type_stack = (struct type_level *)type_stack->base.prev;
 }
 
 /* Some simple list processing predicates.  */
@@ -1021,7 +1011,7 @@ make_memoized_table_entry (type, name, function_p)
       obstack_blank (&type_obstack, sizeof (struct memoized_entry *));
       CLASSTYPE_MTABLE_ENTRY (type) = my_new_memoized_entry (0);
       type_stack->len++;
-      if (type_stack->len * 2 >= type_stack->limit)
+      if (type_stack->len * 2 >= type_stack->base.limit)
 	abort ();
     }
   if (function_p)
@@ -2185,7 +2175,8 @@ dfs_init_vbase_pointers (type)
       tree ref = build (COMPONENT_REF, TREE_TYPE (fields),
 			build_indirect_ref (this_vbase_ptr, 0), fields);
       tree init = CLASSTYPE_SEARCH_SLOT (TREE_TYPE (TREE_TYPE (fields)));
-      vbase_init_result = tree_cons (NULL_TREE, build_modify_expr (ref, NOP_EXPR, init),
+      vbase_init_result = tree_cons (TREE_TYPE (TREE_TYPE (fields)),
+				     build_modify_expr (ref, NOP_EXPR, init),
 				     vbase_init_result);
       fields = TREE_CHAIN (fields);
     }
