@@ -22,6 +22,7 @@ and this notice must be preserved on all copies.  */
 /* Borrow everything that is C from c-tree.h,
    but do so by copy, not by inclusion, since c-tree.h defines
    lang_identifier.  */
+
 /* Definitions for C parsing and type checking.
    Copyright (C) 1987 Free Software Foundation, Inc.
 
@@ -228,7 +229,20 @@ extern int flag_handle_exceptions;
    inline by default.  */
 
 extern int flag_default_inline;
+
+/* Nonzero means recognize and handle exception handling constructs.  */
+
+extern int flag_no_inline;
 
+/* C++ language-specific tree codes.  */
+#define DEFTREECODE(SYM, NAME, TYPE, LENGTH) SYM,
+enum cplus_tree_code {
+  __DUMMY = LAST_AND_UNUSED_TREE_CODE,
+#include "cplus-tree.def"
+  LAST_CPLUS_TREE_CODE
+};
+#undef DEFTREECODE
+
 enum languages { lang_c, lang_cplusplus };
 
 /* Macros to make error reporting functions' lives easier.  */
@@ -668,7 +682,7 @@ struct lang_type
 
 /* For FUNCTION_TYPE or METHOD_TYPE, a list of the exceptions that
    this type can raise.  */
-#define TYPE_RAISES_EXCEPTIONS(NODE) TYPE_NAME (NODE)
+#define TYPE_RAISES_EXCEPTIONS(NODE) TYPE_NONCOPIED_PARTS (NODE)
 
 struct lang_decl_flags
 {
@@ -1004,9 +1018,10 @@ extern char *lang_printable_name ();
 
 /* The largest size a virtual function table can be.
    Must be a (power of 2).  */
-
+#ifndef VINDEX_MAX
 #define VINDEX_MAX ((unsigned)128)
 /* This is the integer ~ (vindex_max - 1).  */
+#endif
 extern tree vtbl_mask;
 
 /* Array type `(void *)[]' */
@@ -1023,6 +1038,7 @@ extern tree get_first_matching_virtual (), get_abstract_virtuals ();
 extern tree build_x_conditional_expr ();
 extern tree merge_component_comparisons ();
 extern tree build_x_unary_op (), build_x_binary_op ();
+extern tree build_component_addr ();
 extern tree build_x_function_call ();
 extern tree build_x_indirect_ref (), build_x_array_ref ();
 extern tree build_x_modify_expr (), build_x_modify_op_expr ();
@@ -1043,9 +1059,11 @@ extern tree basetype_or_else ();
 
 /* in tree.c */
 extern tree build_let ();
+extern tree decl_type_context ();
 
 /* in cplus-tree.c */
 extern tree build1 ();
+extern tree build_cplus_new ();
 extern tree build_cplus_array_type ();
 extern tree build_cplus_method_type ();
 extern tree build_classtype_variant ();
@@ -1103,7 +1121,7 @@ extern tree current_class_name, current_class_type, current_class_decl, C_C_D;
 extern tree current_vtable_decl;
 
 /* in cplus-init.c  */
-extern tree resolve_member_ref ();
+extern tree resolve_offset_ref ();
 extern tree purpose_member (), value_member ();
 extern void check_base_init ();
 extern void do_member_init ();
@@ -1113,7 +1131,7 @@ extern tree current_base_init_list, current_member_init_list;
 extern tree get_linktable_name (), get_dtable_name (), get_sos_dtable ();
 #endif
 extern tree get_member_function ();
-extern tree build_member_call (), build_member_ref ();
+extern tree build_member_call (), build_offset_ref ();
 
 extern int current_function_assigns_this;
 extern int current_function_just_assigned_this;
@@ -1318,6 +1336,7 @@ extern tree default_conversion (), pushdecl (), pushdecl_top_level ();
 extern tree make_instance_name (), do_decl_overload ();
 extern tree maybe_build_cleanup ();
 extern tree build_instantiated_decl (), instantiate_type ();
+extern tree require_instantiated_type ();
 extern tree build_vtbl_ref ();
 extern tree make_anon_parm_name ();
 extern int resolves_to_fixed_type_p ();
@@ -1469,12 +1488,12 @@ extern tree build_with_cleanup ();
 extern char *operator_name_string ();
 
 extern tree get_base_type ();
-extern tree build_op_identifier ();
+extern tree build_opid ();
 
 /* Indexed by TREE_CODE, these tables give C-looking names to
    operators represented by TREE_CODES.  For example,
    opname_tab[(int) MINUS_EXPR] == "-".  */
-extern char *opname_tab[], *assignop_tab[];
+extern char **opname_tab, **assignop_tab;
 
 extern tree build_lang_decl (), build_lang_field_decl ();
 extern tree make_lang_type ();
@@ -1485,7 +1504,7 @@ extern tree convert_from_reference ();
 
 /* in cplus-search.c  */
 extern tree init_vbase_pointers ();
-extern tree build_vbase_pointer ();
+extern tree build_vbase_pointer (), build_vbase_path ();
 extern tree lookup_fnfield (), next_baselink ();
 
 extern tree get_base_type ();
