@@ -1185,7 +1185,7 @@ build_functional_cast (exp, parms)
 	  return rval;
 	if (flags & LOOKUP_NO_CONVERSION)
 	  {
-	    flags = LOOKUP_COMPLAIN;
+	    flags = LOOKUP_NORMAL;
 	    goto try_again;
 	  }
       }
