@@ -666,7 +666,8 @@ copy_for_inline (orig)
   /* Replace this rtx with a copy of itself.  */
 
   x = rtx_alloc (code);
-  bcopy (orig, x, sizeof (int) * (GET_RTX_LENGTH (code) + 1));
+  bcopy (orig, x, (sizeof (*x) - sizeof (x->fld)
+		   + sizeof (x->fld[0]) * GET_RTX_LENGTH (code)));
 
   /* Now scan the subexpressions recursively.
      We can store any replaced subexpressions directly into X
@@ -1349,8 +1350,10 @@ expand_inline_function (fndecl, parms, target, ignore, type, structure_value_add
 	   && (GET_CODE (inline_target) != SUBREG
 	       || SUBREG_REG (inline_target) != target))
     {
-      /* Copy result back to TARGET if TARGET is not INLINE_TARGET.  */
-      emit_move_insn (target, inline_target);
+      /* Copy result back to TARGET if TARGET is not INLINE_TARGET.
+         In general, these should always wind up being the same mode,
+	 after SUBREGs, if any, are stripped.  */
+      convert_move (target, inline_target, 0);
     }
 
   return target;
@@ -1475,7 +1478,13 @@ copy_rtx_and_substitute (orig)
 		 because SUBREG is not required for hard regs.
 		 If not, adjust mode of inline_target to fit the context.  */
 	      if (inline_target == 0)
-		abort ();
+		{
+		  /* If there isn't an inline target, so be it.
+		     Just fake up a reg--it won't get used
+		     for anything important anyway.  */
+		  inline_target = gen_reg_rtx (mode);
+		  return inline_target;
+		}
 	      if (mode == GET_MODE (inline_target))
 		return inline_target;
 	      return gen_rtx (SUBREG, mode, inline_target, 0);
@@ -1771,7 +1780,10 @@ copy_rtx_and_substitute (orig)
       else
 	temp = copy_rtx_and_substitute (copy);
 
-      return change_address (orig, mode, temp);
+      temp = change_address (orig, mode, temp);
+      /* Deals with GCC bug for now.  */
+      RTX_UNCHANGING_P (temp) = 0;
+      return temp;
 
     case RETURN:
       abort ();
